package com.ichaoj.ams.task;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.date.format.DatePrinter;
import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ichaoj.ams.entity.AmsAddressAccount;
import com.ichaoj.ams.entity.AmsTradeRecord;
import com.ichaoj.ams.response.statistics.DailyCostResponse;
import com.ichaoj.ams.response.statistics.NotifyTaskResponse;
import com.ichaoj.ams.service.*;
import com.ichaoj.email.IEmailProvider;
import com.ichaoj.web.util.ParamTemplate;
import com.xxl.job.core.handler.annotation.XxlJob;
import jdk.internal.org.objectweb.asm.tree.FieldInsnNode;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @author gavin
 * @date 2023/6/1
 */
@Component
@RequiredArgsConstructor
@Slf4j
public class NotifyTask {

    private final IAmsTradeRecordService tradeService;

    private final IAmsAddressAccountService accountService;

    private final IEmailProvider emailProvider;
    private final ParamTemplate paramTemplate;

    @XxlJob("ams_weekly_report")
    @SuppressWarnings("all")
    public void statisticsWeeklyReport() {
        NotifyTaskResponse response = new NotifyTaskResponse();

        LocalDateTime startTime = LocalDateTimeUtil.of(DateUtil.lastWeek());
        LocalDateTime endTime = LocalDateTime.now();

        // 执行核心统计方法
        executeCoreMethod(response, startTime, endTime);

        Set<String> emailSet = getEmailSet();
        ThreadUtil.execute(() -> {
            for (String email : emailSet) {
                sendEmail(response, email, startTime, endTime);
            }
        });
    }

    private void executeCoreMethod(NotifyTaskResponse response, LocalDateTime startTime, LocalDateTime endTime) {
        // 统计地址数量和精品号数量
        countAddress(response, startTime, endTime, 0);
        countAddress(response, null, null, 0);
        countAddress(response, startTime, endTime, 1);
        countAddress(response, null, null, 1);

        //  统计交易次数
        countTrade(response, startTime, endTime);
        countTrade(response, null, null);
    }

    @XxlJob("ams_monthly_report")
    @SuppressWarnings("all")
    public void statisticsMonthlyReport() {
        NotifyTaskResponse response = new NotifyTaskResponse();
        LocalDateTime startTime = LocalDateTimeUtil.of(DateUtil.lastMonth());
        LocalDateTime endTime = LocalDateTime.now();

        // 执行核心统计方法
        executeCoreMethod(response, startTime, endTime);

        Set<String> emailSet = getEmailSet();
        ThreadUtil.execute(() -> {
            for (String email : emailSet) {
                sendEmail(response, email, startTime, endTime);
            }
        });
    }

    @NotNull
    private Set<String> getEmailSet() {
        String emailStr = paramTemplate.getParamValue("str_ams_data_notify_email");
        String[] emails = emailStr.split(StrUtil.COMMA);
        Set<String> emailSet = new HashSet<>();
        if (emails.length == 1) {
            emailSet.add(emailStr);
        } else {
            emailSet.addAll(ListUtil.of(emails));
        }
        return emailSet;
    }

    private void countTrade(NotifyTaskResponse response, LocalDateTime startTime, LocalDateTime endTime) {
        long count = tradeService.count(new LambdaQueryWrapper<AmsTradeRecord>()
                .ge(startTime != null, AmsTradeRecord::getCreateTime, startTime)
                .le(endTime != null, AmsTradeRecord::getCreateTime, endTime)
        );
        if (startTime == null) {
            response.setTotalTradeCount(count);
        } else {
            response.setRecentTradeCount(count);
        }
    }

    @SneakyThrows
    private void sendEmail(NotifyTaskResponse response, String email, LocalDateTime startTime, LocalDateTime endTime) {
        log.info("{} 你好，ams空投系统 {} - {} 报表统计，" +
                        "新增：{} 个批量地址,{} 个精品号地址," +
                        "{} 条交易，" +
                        "总计：{} 个批量地址,{} 个精品号地址," +
                        "{} 条交易.", email, startTime, endTime, response.getRecentBatchCount()
                , response.getRecentPremiumCount(), response.getRecentTradeCount(),
                response.getTotalBatchCount(), response.getTotalPremiumCount(), response.getTotalTradeCount());
        emailProvider.sendEmail("ReportEmailTemplate.html",
                "ams统计报表",
                email,
                context -> {
                    context.setVariable("title", StrUtil.format("ams空投系统 {} - {} 报表统计",
                            DateUtil.format(startTime, DatePattern.NORM_DATE_PATTERN),
                            DateUtil.format(endTime, DatePattern.NORM_DATE_PATTERN)));
                    context.setVariable("sendTime", DateUtil.format(new Date(), DatePattern.NORM_DATETIME_PATTERN));
                    context.setVariable("message", response);
                });
    }

    private void countAddress(NotifyTaskResponse response, LocalDateTime startTime, LocalDateTime endTime, int addressType) {
        long count = accountService.count(new LambdaQueryWrapper<AmsAddressAccount>()
                .eq(AmsAddressAccount::getAddressType, addressType)
                .ge(startTime != null, AmsAddressAccount::getCreateTime, startTime)
                .le(endTime != null, AmsAddressAccount::getCreateTime, endTime)
        );
        if (startTime == null) {
            if (addressType == 0) {
                response.setTotalPremiumCount(count);
            } else {
                response.setTotalBatchCount(count);
            }
        } else {
            if (addressType == 0) {
                response.setRecentPremiumCount(count);
            } else {
                response.setRecentBatchCount(count);
            }
        }

    }


}
